@@ -1,10 +1,13 @@
 import os
 import sys
 from copy import copy
+from collections import OrderedDict
 
 import neo
 
-from spykeutils.plugin.data_provider import DataProvider
+from data_provider import DataProvider
+from .. import conversions as convert
+
 
 class NeoDataProvider(DataProvider):
     """ Base class for data providers using NEO"""
@@ -156,42 +159,6 @@ class NeoDataProvider(DataProvider):
 
         return data
 
-    def blocks(self):
-        """ Return a list of selected Block objects
-        """
-        return []
-
-    def segments(self):
-        """ Return a list of selected Segment objects
-        """
-        return []
-
-    def recording_channel_groups(self):
-        """ Return a list of selected
-        """
-        return []
-
-    def recording_channels(self):
-        """ Return a list of selected recording channel indices
-        """
-        return []
-
-    def recording_channel_objects(self):
-        """ Return a list of selected RecordingChannel objects
-        """
-        rcgs = self.recording_channel_groups()
-        channel_ids = self.recording_channels()
-        channels = []
-        for rcg in rcgs:
-            channels.extend([c for c in rcg.recordingchannels
-                             if c.index in channel_ids])
-        return channels
-
-    def units(self):
-        """ Return a list of selected Unit objects
-        """
-        return []
-
     def spike_trains(self):
         """ Return a list of SpikeTrain objects.
         """
@@ -209,7 +176,7 @@ class NeoDataProvider(DataProvider):
         """ Return a dictionary (indexed by Unit) of lists of
         SpikeTrain objects.
         """
-        trains = {}
+        trains = OrderedDict()
         segments = self.segments()
         for u in self.units():
             trains[u] = [t for t in u.spiketrains if t.segment in segments]
@@ -305,7 +272,7 @@ class NeoDataProvider(DataProvider):
         """ Return a dictionary (indexed by Segment) of lists of
         SpikeTrain objects.
         """
-        trains = {}
+        trains = OrderedDict()
         units = self.units()
         for s in self.segments():
             trains[s] = [t for t in s.spiketrains if t.unit in units]
@@ -322,22 +289,22 @@ class NeoDataProvider(DataProvider):
         """ Return a dictionary (indexed by Unit) of dictionaries
         (indexed by Segment) of SpikeTrain objects.
         """
-        trains = {}
+        trains = OrderedDict()
         segments = self.segments()
         for u in self.units():
             for s in segments:
                 segtrains = [t for t in u.spiketrains if t.segment == s]
                 if segtrains:
                     if u not in trains:
-                        trains[u] = {}
+                        trains[u] = OrderedDict()
                     trains[u][s] = segtrains[0]
             nonetrains = [t for t in u.spiketrains if t.segment is None]
             if nonetrains:
                 if u not in trains:
-                    trains[u] = {}
+                    trains[u] = OrderedDict()
                 trains[u][self.no_segment] = nonetrains[0]
 
-        nonetrains = {}
+        nonetrains = OrderedDict()
         for s in self.segments():
             segtrains = [t for t in s.spiketrains if t.unit is None]
             if segtrains:
@@ -364,7 +331,7 @@ class NeoDataProvider(DataProvider):
         """ Return a dictionary (indexed by Unit) of lists of
         Spike objects.
         """
-        spikes = {}
+        spikes = OrderedDict()
         segments = self.segments()
         for u in self.units():
             spikes[u] = [t for t in u.spikes if t.segment in segments]
@@ -381,7 +348,7 @@ class NeoDataProvider(DataProvider):
         """ Return a dictionary (indexed by Segment) of lists of
         Spike objects.
         """
-        spikes = {}
+        spikes = OrderedDict()
         units = self.units()
         for s in self.segments():
             spikes[s] = [t for t in s.spikes if t.unit in units]
@@ -398,22 +365,22 @@ class NeoDataProvider(DataProvider):
         """ Return a dictionary (indexed by Unit) of dictionaries
         (indexed by Segment) of Spike lists.
         """
-        spikes = {}
+        spikes = OrderedDict()
         segments = self.segments()
         for u in self.units():
             for s in segments:
                 segtrains = [t for t in u.spikes if t.segment == s]
                 if segtrains:
                     if u not in spikes:
-                        spikes[u] = {}
+                        spikes[u] = OrderedDict()
                     spikes[u][s] = segtrains
             nonespikes = [t for t in u.spikes if t.segment is None]
             if nonespikes:
                 if u not in spikes:
-                    spikes[u] = {}
+                    spikes[u] = OrderedDict()
                 spikes[u][self.no_segment] = nonespikes
 
-        nonespikes = {}
+        nonespikes = OrderedDict()
         for s in self.segments():
             segspikes = [t for t in s.spikes if t.unit is None]
             if segspikes:
@@ -423,40 +390,72 @@ class NeoDataProvider(DataProvider):
 
         return spikes
 
-    def events(self):
+    def events(self, include_array_events = True):
         """ Return a dictionary (indexed by Segment) of lists of
         Event objects.
         """
-        ret = {}
+        ret = OrderedDict()
         for s in self.segments():
             ret[s] = s.events
+            if include_array_events:
+                for a in s.eventarrays:
+                    ret[s].extend(convert.event_array_to_events(a))
         return ret
 
-    def labeled_events(self, label):
+    def labeled_events(self, label, include_array_events = True):
         """ Return a dictionary (indexed by Segment) of lists of Event
         objects with the given label.
         """
-        ret = {}
+        ret = OrderedDict()
         for s in self.segments():
             ret[s] = [e for e in s.events if e.label == label]
+            if include_array_events:
+                for a in s.eventarrays:
+                    events = convert.event_array_to_events(a)
+                    ret[s].extend((e for e in events if e.label == label))
         return ret
 
-    def epochs(self):
+    def event_arrays(self):
+        """ Return a dictionary (indexed by Segment) of lists of
+        EventArray objects.
+        """
+        ret = OrderedDict()
+        for s in self.segments():
+            ret[s] = s.eventarrays
+        return ret
+
+    def epochs(self, include_array_epochs = True):
         """ Return a dictionary (indexed by Segment) of lists of
         Epoch objects.
         """
-        ret = {}
+        ret = OrderedDict()
         for s in self.segments():
             ret[s] = s.epochs
+            if include_array_epochs:
+                for a in s.epocharrays:
+                    ret[s].extend(convert.epoch_array_to_epochs(a))
         return ret
 
-    def labeled_epochs(self, label):
+    def labeled_epochs(self, label, include_array_epochs = True):
         """ Return a dictionary (indexed by Segment) of lists of Epoch
         objects with the given label.
         """
-        ret = {}
+        ret = OrderedDict()
         for s in self.segments():
             ret[s] = [e for e in s.epochs if e.label == label]
+            if include_array_epochs:
+                for a in s.epocharrays:
+                    epochs = convert.epoch_array_to_epochs(a)
+                    ret[s].extend((e for e in epochs if e.label == label))
+        return ret
+
+    def epoch_arrays(self):
+        """ Return a dictionary (indexed by Segment) of lists of
+        EpochArray objects.
+        """
+        ret = OrderedDict()
+        for s in self.segments():
+            ret[s] = s.epocharrays
         return ret
 
     def num_analog_signals(self):
@@ -471,7 +470,8 @@ class NeoDataProvider(DataProvider):
         channels = self.recording_channels()
         for s in self.segments():
             signals.extend([t for t in s.analogsignals
-                           if t.recordingchannel in channels or t.recordingchannel is None])
+                           if t.recordingchannel in channels or
+                              t.recordingchannel is None])
         for u in self.recording_channels():
             signals.extend([t for t in u.analogsignals if t.segment is None])
 
@@ -481,11 +481,12 @@ class NeoDataProvider(DataProvider):
         """ Return a dictionary (indexed by Segment) of lists of
         AnalogSignal objects.
         """
-        signals = {}
+        signals = OrderedDict()
         channels = self.recording_channels()
         for s in self.segments():
             signals[s] = [t for t in s.analogsignals
-                          if t.recordingchannel in channels]
+                          if t.recordingchannel in channels or
+                             t.recordingchannel is None]
 
         nonesignals = []
         for c in channels:
@@ -500,11 +501,12 @@ class NeoDataProvider(DataProvider):
         """ Return a dictionary (indexed by RecordingChannel) of lists
         of AnalogSignal objects.
         """
-        signals = {}
+        signals = OrderedDict()
         segments = self.segments()
         for c in self.recording_channels():
             signals[c] = [t for t in c.analogsignals
-                          if t.segment in segments]
+                          if t.segment in segments or
+                             t.segment is None]
 
         nonesignals = []
         for s in segments:
@@ -519,22 +521,22 @@ class NeoDataProvider(DataProvider):
         """ Return a dictionary (indexed by RecordingChannel) of
         dictionaries (indexed by Segment) of AnalogSignal lists.
         """
-        signals = {}
+        signals = OrderedDict()
         segments = self.segments()
         for c in self.recording_channels():
             for s in segments:
                 segsignals = [t for t in c.analogsignals if t.segment == s]
                 if segsignals:
                     if c not in signals:
-                        signals[c] = {}
+                        signals[c] = OrderedDict()
                     signals[c][s] = segsignals
             nonesignals = [t for t in c.analogsignals if t.segment is None]
             if nonesignals:
                 if c not in signals:
-                    signals[c] = {}
+                    signals[c] = OrderedDict()
                 signals[c][self.no_segment] = nonesignals
 
-        nonesignals = {}
+        nonesignals = OrderedDict()
         for s in self.segments():
             segsignals = [t for t in s.analogsignals
                           if t.recordingchannel is None]
@@ -569,11 +571,11 @@ class NeoDataProvider(DataProvider):
         """ Return a dictionary (indexed by Segment) of lists of
         AnalogSignalArray objects.
         """
-        signals = {}
+        signals = OrderedDict()
         channelgroups = self.recording_channel_groups()
         for s in self.segments():
             signals[s] = [t for t in s.analogsignalarrays
-                          if t.recordingchannel in channelgroups]
+                          if t.recordingchannelgroup in channelgroups]
 
         nonesignals = []
         for c in channelgroups:
@@ -588,7 +590,7 @@ class NeoDataProvider(DataProvider):
         """ Return a dictionary (indexed by RecordingChannelGroup) of
         lists of AnalogSignalArray objects.
         """
-        signals = {}
+        signals = OrderedDict()
         segments = self.segments()
         for c in self.recording_channel_groups():
             signals[c] = [t for t in c.analogsignalarrays
@@ -607,7 +609,7 @@ class NeoDataProvider(DataProvider):
         """ Return a dictionary (indexed by RecordingChannelGroup) of
         dictionaries (indexed by Segment) of AnalogSignalArray objects.
         """
-        signals = {}
+        signals = OrderedDict()
         segments = self.segments()
         for c in self.recording_channel_groups():
             for s in segments:
@@ -615,16 +617,16 @@ class NeoDataProvider(DataProvider):
                               if t.segment == s]
                 if segsignals:
                     if c not in signals:
-                        signals[c] = {}
+                        signals[c] = OrderedDict()
                     signals[c][s] = segsignals[0]
             nonesignals = [t for t in c.analogsignalarrays
                            if t.segment is None]
             if nonesignals:
                 if c not in signals:
-                    signals[c] = {}
+                    signals[c] = OrderedDict()
                 signals[c][self.no_segment] = nonesignals[0]
 
-        nonesignals = {}
+        nonesignals = OrderedDict()
         for s in self.segments():
             segsignals = [t for t in s.analogsignalarrays
                           if t.recordingchannelgroup is None]
